@@ -1,13 +1,13 @@
 import dataclasses
 from enum import Enum
-from typing import List, Optional, Type, TypeVar
+from typing import Callable, Optional, TypeVar
+from typing_extensions import dataclass_transform
 
-from strawberry.object_type import _wrap_dataclass
+from strawberry.types.field import StrawberryField, field
+from strawberry.types.object_type import _wrap_dataclass
 from strawberry.types.type_resolver import _get_fields
 
 from .directive import directive_field
-from .field import StrawberryField, field
-from .utils.typing import __dataclass_transform__
 
 
 class Location(Enum):
@@ -28,36 +28,43 @@ class Location(Enum):
 class StrawberrySchemaDirective:
     python_name: str
     graphql_name: Optional[str]
-    locations: List[Location]
-    fields: List["StrawberryField"]
+    locations: list[Location]
+    fields: list["StrawberryField"]
     description: Optional[str] = None
     repeatable: bool = False
+    print_definition: bool = True
+    origin: Optional[type] = None
 
 
-T = TypeVar("T", bound=Type)
+T = TypeVar("T", bound=type)
 
 
-@__dataclass_transform__(
-    order_default=True, field_descriptors=(directive_field, field, StrawberryField)
+@dataclass_transform(
+    order_default=True,
+    kw_only_default=True,
+    field_specifiers=(directive_field, field, StrawberryField),
 )
 def schema_directive(
     *,
-    locations: List[Location],
+    locations: list[Location],
     description: Optional[str] = None,
     name: Optional[str] = None,
     repeatable: bool = False,
-):
+    print_definition: bool = True,
+) -> Callable[[T], T]:
     def _wrap(cls: T) -> T:
-        cls = _wrap_dataclass(cls)
-        fields = _get_fields(cls)
+        cls = _wrap_dataclass(cls)  # type: ignore
+        fields = _get_fields(cls, {})
 
-        cls.__strawberry_directive__ = StrawberrySchemaDirective(
+        cls.__strawberry_directive__ = StrawberrySchemaDirective(  # type: ignore[attr-defined]
             python_name=cls.__name__,
             graphql_name=name,
             locations=locations,
             description=description,
             repeatable=repeatable,
             fields=fields,
+            print_definition=print_definition,
+            origin=cls,
         )
 
         return cls
